#-
* This file is part of aITALC
* Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>
*------------------------------------------------------------- 
** <joinalternativemm.frm> designed to pick up the kinematical information out of DIANA and
** reproduce the Matrix Elements and differential cross section subroutines.
** It calculates explicitely the product of 2 fermion chains.
*-------------------------------------------------------------------------------
#include Kit.inc
#include model.inc
#include aITALC.inc
#include `processname'.in #exts1
#include `processname'.in #topologies0
#define code "15"

NF FER,FERB;
auto S m,u;
CF Hb,Hc,Hq;
S [S],[T];
** For calibrating contraction coefficients
S oX;

#if (`LEGS' == 4 ) || ( `LEGS' == 3)
*-------------------------------------------------------------------------------
* Leaving the possibility to work in a Right-Left instead of Gamma,Gamma5
#if `code' == 15
  #define a "1"
  #define b "5"
#elseif `code' == lr
  #define a "l"
  #define b "r"
#endif
*-------------------------------------------------------------------------------
#define elements22 "2"
#define elements11 "2"
#define elements21 "1"
#define elements12 "1"
#define elements10 "1"
#define elements01 "1"
#define elements00 "1"
** moreover there are no elements for other combinations
#define elements02 "0"
#define elements20 "0"

* Let's define the big chains, for all possible fermion channels
** The optimal is of course to be able to generate as many chains as needed (each need two indices)
** for the Hb and Hconjugated terms. This requires inserting arbitrary number of #do loops, what is
** not suported in form (should read something like #include doloopchains. and #include enddoloopchains)

#do ch1={`fermioncurrentsnames'}
 #do ch2={`fermioncurrentsnames'}
  #do i11=0,2
   #do i21=0,2
    #do eli=1,`elements`i11'`i21''
     #do j11=0,2
      #do j21=0,2
       #do elj=1,`elements`j11'`j21''
        #do i12=-1,1,2
         #do i22=-1,1,2
          #do j12=-1,1,2
           #do j22=-1,1,2
            L [`ch1'`ch2',`i11',`i12',`i21',`i22',`eli',`j11',`j12',`j21',`j22',`elj']=
            Hc(`ch1',`i11',`i12',`i21',`i22',`eli')*Hb(`ch2',`j11',`j12',`j21',`j22',`elj');
           #enddo
          #enddo
         #enddo
        #enddo
       #enddo
      #enddo
     #enddo
    #enddo
   #enddo
  #enddo
 #enddo
#enddo

*print ;
*.end


** The FER and FERB functions will represent the spinors of fermions
** They have 3 arguments, 1st is the fermion line they belong to,
** 2nd is a variable telling us whether it is fermion (antifermion)
** +1 (-1). Last one is the mass.

** Let's make the index expansion of different kind of matrix elements
** Hb and Hc will serve as channel-line labels to avoid pure-pure substitutions when tracing
** an unique line over mixed channels.
** The function Nj keep trace of the ternsor-rank order of the elements, required to apply
** the correct factor when contraction, as appearing in eq (1.15) of hep-ph/0306087
#do ch={`fermioncurrentsnames'}
 #do ic=1,`EXTFERMICURRENTS'
  #redefine ipart "`IFFI`ch'{`ic'}'"
  id Hc(`ch',a11?,a12?,?a)=FER(`ic',u{`ipart'},m{`ipart'})*FERB(`ic',u{`ipart'},m{`ipart'})*
                           Gc(`ic',a11,a12,be`ic'1,be`ic'2)*Hc(`ch',?a);
 #enddo
** We have splitted the 2 do loops over ic, to avoid substitutions at the same time of two ic's
 #do ic=1,`EXTFERMICURRENTS'
  #redefine ipart "`IFFO`ch'{`ic'}'"
  id Hb(`ch',a11?,a12?,?a)=FER(`ic',u{`ipart'},m{`ipart'})*FERB(`ic',u{`ipart'},m{`ipart'})*
                           Gb(`ic',a11,a12,al`ic'1,al`ic'2)*Hb(`ch',?a);
  #enddo
#enddo



*.sort
*L dmy=1;
*print [TS,2,1,2,1,2,2,1,2,1,2],[TT,2,1,2,1,2,2,1,2,1,2];
*.end
*-------------------------------------------------------------------------------


** Shorthand for the inner spinors
#define FFQ "FER(2,u4?,m4?)*FERB(2,u4?,m4?)"
#define FF "FER(2,u4,m4)*FERB(2,u4,m4)"

** Substitutions of the matrix elements in accordance with joinalternativefofa
#do a ={b,c}
 #if `a' == b
  #redefine al "al"
 #elseif `a' == c
  #redefine al "be"
 #else
  #message[joinalternativemm: Are there three chains of gamma matrices??? I give up!]
  exit;
 #endif

* Since the indices in conjugated terms are labelled by be, (those in Hb are labelled by al),
* we can keep trace for later subst.
* The Function Hq only retains the channel for evaluation of traces
id G`a'(1,2,?a1)*`FFQ'*G`a'(2,2,?a2)*H`a'(T?,1)=Mom(T,2,-1,`al'12)*Mom(T,1,-1,`al'22)*Delta(`al'11,`al'21)*
                                                G`a'(1,2,?a1)*`FF'*G`a'(2,2,?a2)*Hq(T);
al G`a'(1,2,?a1)*`FFQ'*G`a'(2,2,?a2)*H`a'(T?,2)=Delta(`al'11,`al'21)*Delta(`al'12,`al'22)*
                                                G`a'(1,2,?a1)*`FF'*G`a'(2,2,?a2)*Hq(T);

al G`a'(1,1,?a1)*`FFQ'*G`a'(2,1,?a2)*H`a'(T?,1)=Mom(T,2,-1,`al'11)*Mom(T,1,-1,`al'21)*
                                                G`a'(1,1,?a1)*`FF'*G`a'(2,1,?a2)*Hq(T);
al G`a'(1,1,?a1)*`FFQ'*G`a'(2,1,?a2)*H`a'(T?,2)=Delta(`al'11,`al'21)*
                                                G`a'(1,1,?a1)*`FF'*G`a'(2,1,?a2)*Hq(T);


al G`a'(1,2,?a1)*`FFQ'*G`a'(2,1,?a2)*H`a'(T?,1)=Mom(T,2,-1,`al'12)*Delta(`al'11,`al'21)*
                                                G`a'(1,2,?a1)*`FF'*G`a'(2,1,?a2)*Hq(T);

al G`a'(1,1,?a1)*`FFQ'*G`a'(2,2,?a2)*H`a'(T?,1)=Mom(T,2,-1,`al'22)*Delta(`al'11,`al'21)*
                                                G`a'(1,1,?a1)*`FF'*G`a'(2,2,?a2)*Hq(T);

al G`a'(1,1,?a1)*`FFQ'*G`a'(2,0,?a2)*H`a'(T?,1)=Mom(T,2,-1,`al'11)*
                                                G`a'(1,1,?a1)*`FF'*G`a'(2,0,?a2)*Hq(T);

al G`a'(1,0,?a1)*`FFQ'*G`a'(2,1,?a2)*H`a'(T?,1)=Mom(T,2,-1,`al'21)*
                                                G`a'(1,0,?a1)*`FF'*G`a'(2,1,?a2)*Hq(T);


al G`a'(1,0,?a1)*`FFQ'*G`a'(2,0,?a2)*H`a'(T?,1)=1*
                                                G`a'(1,0,?a1)*`FF'*G`a'(2,0,?a2)*Hq(T);
#enddo

*print [TS,2,1,2,1,2,2,1,2,1,2],[TT,2,1,2,1,2,2,1,2,1,2];
*.end
.sort

*-------------------------------------------------------------------------------
** Commutation of different fermion lines means we have to order them, before
** equaling traces, otherwise mixing channel terms who belong to iline=0 or fermicurrents+1
** show up a wrong ordering.

** We avoid possible substituions in Delta (error because of antisymmetry) or Mom
repeat;
id FER?{Gb,Gc,FER,FERB}(2,?a)*FERB?{Gb,Gc,FER,FERB}(1,?b)= FERB(1,?b)*FER(2,?a);
endrepeat;

*print [ST,2,1,2,1,2,2,1,2,1,2],[TT,2,1,2,1,2,2,1,2,1,2];
*.end
.sort

*-------------------------------------------------------------------------------
** The main sentence for all functions with index line and 3 ARGUMENTS
if (match(Hq(T?)*Hq(T?)) == 0);
#do j=1,`EXTFERMICURRENTS'
 id FER?{Gb,Gc,FER,FERB}(`j',?a)=FER({`EXTFERMICURRENTS'+1},?a);
#enddo
endif;
id Hq(S?)=1;

** Now time for evaluation of the contraction coefficients;
*#do a={b,c}
* if (match(G`a'(iline?,2,?a)));
*  Multiply oX;
* endif;
*#enddo

*print  [TS,2,1,2,1,2,1,1,1,1,1],[TT,1,1,2,-1,1,2,1,1,1,1];
*.end
.sort
*-------------------------------------------------------------------------------
*unhide;
*drop dmy;
** Color sum and average on states (without color (gluons) exchange)
** For formulae, check page 142/143 of alorca's doctorand book "Loop's and techniques"
** Basically is an extension of the multiplicative color factor in the cross section.

if (match(FER({`EXTFERMICURRENTS'+1},u1?,m1?)));
* Case of interference; combinatorial factor = 1/(color any ext fermion)
 multiply 1/`color1';
elseif (match(FER(iline?$finalline3,u3,m3)) && match(FER(iline?$finalline4,u4,m4)));
 if ($finalline3 == $finalline4);
* Case of SSdagger, since they (particles 3,4) share the same fermion line index
* and it's different from 0 (reserved for interference terms)  
  multiply `color3'/`color1';
 else;
* Case of TTdagger and UUdagger, they multiply by 1 for all cases.
 endif;
endif;


** Spin average and sum
id FER(iline?,u1,m1)*FERB(iline?,u1,m1)=1/2*(g_(iline,`absep1')+u1*gi_(iline)*m1);
id FER(iline?,u2,m2)*FERB(iline?,u2,m2)=1/2*(g_(iline,`absep2')+u2*gi_(iline)*m2);
** The minus before the momentum is because outgoing convention
id FER(iline?,u3,m3)*FERB(iline?,u3,m3)=  -1*g_(iline,`absep3')+u3*gi_(iline)*m3 ;
id FER(iline?,u4,m4)*FERB(iline?,u4,m4)=  -1*g_(iline,`absep4')+u4*gi_(iline)*m4 ;

** If we want them inmediately to be active for printing sort them,
.sort
*-------------------------------------------------------------------------------

** Go for the substitions of the Gamma chains in pure channel
#do ic=1,{`EXTFERMICURRENTS'+1}
 #do i12=-1,1,2
** The standard projector
  #redefine gP "g{7-{`i12'+1}/2}_"
** The resulted adjoint
  #redefine gA "g{6+{`i12'+1}/2}_"

* The adjoints
** Because Form has already a g6_=gi_+g5_ and g7_=gi_-g5_
 id Gc(`ic',0,`i12',al11?,al12?)=1/2*(`gP'(`ic'));
 id Gc(`ic',1,`i12',al11?,al12?)=1/2*(`gA'(`ic'))*g_(`ic',al11);
 id Gc(`ic',2,`i12',al11?,al12?)=1/2*(`gP'(`ic'))*i_/2*g_(`ic',al11,al12);
* id Gc(`ic',0,`i12',al11?,al12?)=1/2*(gi_(`ic')-a1*g5_(`ic'));
* id Gc(`ic',1,`i12',al11?,al12?)=1/2*(gi_(`ic')+a1*g5_(`ic'))*g_(`ic',al11);
* id Gc(`ic',2,`i12',al11?,al12?)=1/2*(gi_(`ic')-a1*g5_(`ic'))*i_/2*g_(`ic',al11,al12);
** Because Form has already a g6_=gi_+g5_ and g7_=gi_-g5_
 id Gb(`ic',0,`i12',al11?,al12?)=1/2*(`gP'(`ic'));
 id Gb(`ic',1,`i12',al11?,al12?)=1/2*(`gP'(`ic'))*g_(`ic',al11);
 id Gb(`ic',2,`i12',al11?,al12?)=1/2*(`gP'(`ic'))*i_/2*g_(`ic',al11,al12);
* id Gb(`ic',0,`i12',al11?,al12?)=1/2*(gi_(`ic')+a1*g5_(`ic'));
* id Gb(`ic',1,`i12',al11?,al12?)=1/2*(gi_(`ic')+a1*g5_(`ic'))*g_(`ic',al11);
* id Gb(`ic',2,`i12',al11?,al12?)=1/2*(gi_(`ic')+a1*g5_(`ic'))*i_/2*g_(`ic',al11,al12);
 #enddo
#enddo

** And the definitions of Delta and composition of momenta.
id Delta(al11?,al21?)=d_(al11,al21);

** The difference of momenta is composed by substracting the moment
#do ch={`fermioncurrentsnames'}
 #do ic=1,`EXTFERMICURRENTS'
  id Mom(`ch',`ic',a1?,be11?)= `absep`IFFI`ch'`ic'''(be11) +a1*(`absep`IFFO`ch'`ic'''(be11));
 #enddo
#enddo

*print +s [ST,1,1,0,-1,1,2,1,2,1,2];
*.end
.sort
*-------------------------------------------------------------------------------
** Now we trace the different fermion lines
#do ic=1,{`EXTFERMICURRENTS'+1}
 Trace4 `ic';
#enddo
*-------------------------------------------------------------------------------
** Because the introduction of epsilon tensor e_ is done automatically, we
** don't have to care about i factors or so, we just contract them

contract;
** Now epsilon vector contractions for the 4 non-indepentent momenta:
id e_(p1,p2,p3,p4)=0;

** We also apply dimensionality, since the ffactors are UV finite by themselves
** after summing the different topologies
id D=4;

*print +s [ST,1,1,0,-1,1,2,1,2,1,2];
.sort
*.end
*-------------------------------------------------------------------------------
** The momenta renaming into Mandelstam

id p1?.p2? = sp(p1,p2);

#do j=1,4
  id sp(`absep{`j'}',`absep{`j'}')=m`j'^2;
#enddo

id sp(`absep1',`absep2')=1/2*(sman-m1^2-m2^2);
id sp(`absep1',`absep3')=1/2*(tman-m1^2-m3^2);
id sp(`absep1',`absep4')=1/2*(uman-m1^2-m4^2);
id sp(`absep2',`absep1')=1/2*(sman-m2^2-m1^2);
id sp(`absep2',`absep3')=1/2*(uman-m2^2-m3^2);
id sp(`absep2',`absep4')=1/2*(tman-m2^2-m4^2);
id sp(`absep3',`absep1')=1/2*(tman-m3^2-m1^2);
id sp(`absep3',`absep2')=1/2*(uman-m3^2-m2^2);
id sp(`absep3',`absep4')=1/2*(sman-m3^2-m4^2);
id sp(`absep4',`absep1')=1/2*(uman-m4^2-m1^2);
id sp(`absep4',`absep2')=1/2*(tman-m4^2-m2^2);
id sp(`absep4',`absep3')=1/2*(sman-m4^2-m3^2);

** And some convention about which variable should be dropped out!

#if (({`fermioncurrentsnames'} == {S,T}) || ({`fermioncurrentsnames'} == {S})   || ({`fermioncurrentsnames'} == {T}))
  id uman=-sman-tman+m1^2+m2^2+m3^2+m4^2;
#elseif (({`fermioncurrentsnames'} == {S,U}) || ({`fermioncurrentsnames'} == {U}))
  id tman=-sman-uman+m1^2+m2^2+m3^2+m4^2;
#elseif {`fermioncurrentsnames'} == {T,U}
  id sman=-tman-uman+m1^2+m2^2+m3^2+m4^2;
#endif

.sort
*-------------------------------------------------------------------------------
** Finally we identify the abstract m1..m4, u1..u4

#do ipart=1,`LEGS'
  id m`ipart'=`em{`ipart'}';
  id u`ipart'=`u{`ipart'}';
#enddo

** Possibility to neglect some masses defined in Neglectedmasses.inc
#call neglectmass(0)


format rational spaces;
print +s +f;
.sort

** The doublefortran format is buggy and conflicts with do loop, for this reason
** we apply our own style for removing 1/16 (the lowest observed) factors

*multiply o16*16;
*id i_*o16=im*o16;
*format doublefortran;
if (match(i_)) multiply -im*i_;
b o16,im;
format doublefortran;
.sort
*-------------------------------------------------------------------------------

#do ch1={`fermioncurrentsnames'}
 #do ch2={`fermioncurrentsnames'}
#write <matrixelements`ch1'`ch2'.F> "C This file <matrixelements`ch1'`ch2'.F> is part of aITALC"
#write <matrixelements`ch1'`ch2'.F> "C Generated automatically with FORM3"
#write <matrixelements`ch1'`ch2'.F> "C Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>"
#write <matrixelements`ch1'`ch2'.F> "C ----------------------------------------------------------------------"
#write <matrixelements`ch1'`ch2'.F> "      subroutine M_`ch1'`ch2'(mm_`ch1'`ch2')"
#write <matrixelements`ch1'`ch2'.F> ""
#write <matrixelements`ch1'`ch2'.F> "      implicit none"
#write <matrixelements`ch1'`ch2'.F> "      integer i1,i2,j1,j2,eli,elj"
#write <matrixelements`ch1'`ch2'.F> "      double complex mm_`ch1'`ch2'(6,6,2,6,6,2)"
#write <matrixelements`ch1'`ch2'.F> "#include <variablelist.hf>"

#write <matrixelements`ch1'`ch2'.F> "C ----------------------------------------------------------------------"
#write <matrixelements`ch1'`ch2'.F> "CC Initializing to 0"
#write <matrixelements`ch1'`ch2'.F> "      DO i1=1,6"
#write <matrixelements`ch1'`ch2'.F> "        DO i2=1,6"
#write <matrixelements`ch1'`ch2'.F> "          DO eli=1,2"
#write <matrixelements`ch1'`ch2'.F> "            DO j1=1,6"
#write <matrixelements`ch1'`ch2'.F> "              DO j2=1,6"
#write <matrixelements`ch1'`ch2'.F> "                DO elj=1,2"
#write <matrixelements`ch1'`ch2'.F> "                  mm_`ch1'`ch2'(i1,i2,eli,j1,j2,elj)= DCMPLX(0)"
#write <matrixelements`ch1'`ch2'.F> "                ENDDO"
#write <matrixelements`ch1'`ch2'.F> "              ENDDO"
#write <matrixelements`ch1'`ch2'.F> "            ENDDO"
#write <matrixelements`ch1'`ch2'.F> "          ENDDO"
#write <matrixelements`ch1'`ch2'.F> "        ENDDO"
#write <matrixelements`ch1'`ch2'.F> "      ENDDO"
#write <matrixelements`ch1'`ch2'.F> "C ----------------------------------------------------------------------"

  #do i11=0,2
   #do i21=0,2
    #do eli=1,`elements`i11'`i21''
     #do j11=0,2
      #do j21=0,2
       #do elj=1,`elements`j11'`j21''
        #do i12=-1,1,2
         #do i22=-1,1,2
          #do j12=-1,1,2
           #do j22=-1,1,2
#write <matrixelements`ch1'`ch2'.F> "      mm_`ch1'`ch2'({1+{2*`i11'}+{1+`i12'}/2},{1+{2*`i21'}+{1+`i22'}/2},`eli',{1+{2*`j11'}+{1+`j12'}/2},{1+{2*`j21'}+{1+`j22'}/2},`elj')=%e",[`ch1'`ch2',`i11',`i12',`i21',`i22',`eli',`j11',`j12',`j21',`j22',`elj'](mm_`ch1'`ch2'({1+{2*`i11'}+{1+`i12'}/2},{1+{2*`i21'}+{1+`i22'}/2},`eli',{1+{2*`j11'}+{1+`j12'}/2},{1+{2*`j21'}+{1+`j22'}/2},`elj'))
           #enddo
          #enddo
         #enddo
        #enddo
       #enddo
      #enddo
     #enddo
    #enddo
   #enddo
  #enddo
#write <matrixelements`ch1'`ch2'.F> "C ----------------------------------------------------------------------"
#write <matrixelements`ch1'`ch2'.F> ""
#write <matrixelements`ch1'`ch2'.F> "      return"
#write <matrixelements`ch1'`ch2'.F> "      end"
#close <matrixelements`ch1'`ch2'.F>
 #enddo
#enddo


*#elseif `LEGS' == 2
*** For selfenergies we don't build up anything
* #write <matrixelements.F> ""
*#elseif `LEGS' == 5
* #write <matrixelements.F> ""
*#else
* #message [joinalternativemm: case not considered]
* exit;
*#endif

.end

*$Id: joinalternativemm.frm,v 1.3 2006/09/26 10:38:07 alorca Exp $
